"""
Python code for highlighting the contributing areas (CAs) & dispersal areas (DAs) of total phosphorus 
(TP) and total nitrogen (TN) export coefficient (EC) hotspots

This is intended to be used as a supplemental post-processing code for the i-Tree Buffer tool, which 
generates ArcGIS raster files of pollutant EC values** for an area of interest. The Buffer output files 
required to run hotspots.py should already be provided in the Buffer_results folder.

SOFTWARE REQUIREMENTS:
~ Python 2.7
  ~ NumPy - array generation & processing
  ~ Pandas - data frame generation & processing
~ ArcMap 10, ArcGIS Spatial Analyst Extension

INPUTS:
~ Buffer_results folder: [will contain all required output rasters from i-Tree Buffer]
  ~ dem: Digital Elevation Model raster of the watershed
  ~ ec_tp_wtd: Raster of annual dynamic export coefficient (ECd) values** for total phosphorus (TP) 
  ~ ec_tn_wtd: raster of annual dynamic export coefficient (ECd) values** for total nitrogen (TN)

OUTPUTS:
~ Point shapefile for each TP & TN hotspot
~ Rasters of CAs and DAs for each TP hotspot
~ Rasters of CAs and DAs for each TN hotspot
~ Raster of river cells
  ~ Based on Arc Hydro GIS suggestion: 1% of the watershed-maximum flow accumulation value
~ Table of ECd** and latitude/longitude values for each hotspot
  ~ Values are identical to Table 2.2 from the Forest Planning report
~ Copies of the original input rasters from Buffer, with their geographic projections all converted 
  from UTM to GCS_North_American_1983
  ~ dem -> dem_gcs
  ~ ec_tp_wtd -> ECd_TP
  ~ ec_tn_wtd -> ECd_TN

OTHER NOTES:
**EC and ECd are used synonymously in this script. For a detailed explanation of the difference between
the two terms, see Chapter 2 of the Forest Planning report. EC & ECd values are in units of kilograms
per hectare per year (kg/ha/yr).

For larger watersheds (rasters >100 MB in size), this code can take over an hour or several hours to 
complete depending on the number of hotspots requested. Plan to allocate computing time and computer 
memory accordingly before attempting.

Author: James Kruegler <james.kruegler@davey.com>
"""
##############################
# IMPORTING DATA & ENVIRONMENT SETTINGS
##############################
# Importing needed modules
import os, csv, math
import shutil
import arcpy
from arcpy.sa import *
from arcpy import env
import numpy as np
import pandas as pd

# Determining the number** of hotspots that will be identified and analyzed for both TP and TN
# **This must be a positive (non-zero) whole number
number_of_hotspots = input("How many hotspots would you like to identify for each pollutant? \n")
if str(type(number_of_hotspots)) != "<type 'int'>":
    print("Invalid answer. Please make sure the number of hotspots is a positive whole number.")
    exit()

# Retrieving the ArcGIS Spatial Analyst license
arcpy.CheckOutExtension("Spatial")

# Setting path name variables
path = os.getcwd()
buffer_rpath = path+'\\Buffer_results'
hotspots_rpath = path+'\\Hotspots_results'

# Clearing old results and (re-)making the results folder
if os.path.exists(hotspots_rpath):
    shutil.rmtree(hotspots_rpath)
    print("Deleted old Hotspots results")
os.makedirs(hotspots_rpath)

# Copying Buffer result rasters from buffer_rpath to hotspots_rpath
arcpy.CopyRaster_management(buffer_rpath+'\\dem', hotspots_rpath+'\\dem')
arcpy.CopyRaster_management(buffer_rpath+'\\ec_tp_wtd', hotspots_rpath+'\\ec_tp_wtd')
arcpy.CopyRaster_management(buffer_rpath+'\\ec_tn_wtd', hotspots_rpath+'\\ec_tn_wtd')
print("Copied needed Buffer results from '"+buffer_rpath+"' to '"+hotspots_rpath+"'")

# Setting the default workspace for geoprocessing
env.workspace = hotspots_rpath
env.overwriteOutput = True
print("Workspace set to "+hotspots_rpath)

##############################
# DATA PREP FOR FUNCTIONS
##############################
# Converting DEM raster from UTM to GCS_North_American_1983 projection
arcpy.ProjectRaster_management("dem", "dem_gcs", out_coor_system=arcpy.SpatialReference(4269))
# Taking measurements of the new DEM_GCS raster
dem_cell_size = Raster('dem_gcs').meanCellWidth
dem_origin_point = Raster('dem_gcs').extent.lowerLeft
dem_origin = "%s %s" % (dem_origin_point.X, dem_origin_point.Y)

# Converting ECd rasters from UTM to GCS_North_American_1983 projection
# Measurements from DEM raster are used as references to ensure all output rasters will have the same cell size and extent
arcpy.ProjectRaster_management("ec_tp_wtd", "ECd_TP", out_coor_system=arcpy.SpatialReference(4269), cell_size=dem_cell_size, Registration_Point=dem_origin)
arcpy.ProjectRaster_management("ec_tn_wtd", "ECd_TN", out_coor_system=arcpy.SpatialReference(4269), cell_size=dem_cell_size, Registration_Point=dem_origin)
print("All input rasters converted to the projection "+str(arcpy.SpatialReference(4269).name))

# Processing the DEM to create rasters of elevation-dependent flow data
arcpy.gp.Fill_sa("dem_gcs", "fill") # Fills sinks (cells with an undefined drainage direction) in the DEM to remove small imperfections in the data
arcpy.gp.FlowDirection_sa('fill','flowdir', "FORCE") # Creates a raster of flow direction from each cell to its downslope neighbor(s)
arcpy.gp.FlowAccumulation_sa('flowdir','flowaccum', "", "FLOAT") # Creates a raster of accumulated flow to each cell, as determined by accumulating weight for all cells that flow into each downslope cell.

# Calculating the threshold value for designating river cells
# Based on Arc Hydro GIS suggestion: 1% of watershed-max FlowAccum value as a starting point
RiverThreshold_FlowAccum = float(arcpy.GetRasterProperties_management('flowaccum', "MAXIMUM").getOutput(0))*0.01
# Creating a "river" raster based on the RiverThreshold_FlowAccum value
river = Con('flowaccum', 0, 1, "VALUE > " + str(RiverThreshold_FlowAccum)) # River cells will have value of 0, non-river cells will have value of 1
river.save('river')
print("Generated rasters of flow direction, flow accumulation, and river cells")

# Converting flow direction and river rasters into NumPy arrays
# Makes the raster values easier to navigate with [row, column] pairs
flowdir_array = arcpy.RasterToNumPyArray('flowdir')
river_array = arcpy.RasterToNumPyArray('river')
print("Generated NumPy arrays for flow direction and river cell rasters")

# Setting up dictionaries to help navigate the flow direction and river NumPy arrays
# Dictionary to look up flow direction value, moving upslope from current cell
# (neighbor cell orientation):(flowdir value needed to move from neighbor cell to current cell)
# Neighbor cell orientation values: 1 for the cell to the immediate right of the current cell; add 1 for each move clockwise through neighbor cells
ca_flowdir_dict = {1:16, 2:32, 3:64, 4:128, 5:1, 6:2, 7:4, 8:8}

# Dictionary of (orientation):([change in row, change in column]) pairs
ca_neighbor_dict = {1:[0,1], 2:[1,1], 3:[1,0], 4:[1,-1], 5:[0,-1], 6:[-1,-1], 7:[-1,0], 8:[-1,1]}

# Dictionary to look up flow direction value, moving downslope from current cell
# (flowdir_array value):([change in row, change in col])
da_flowdir_dict = {1:[0,1], 2:[1,1], 4:[1,0], 8:[1,-1], 16:[0,-1], 32:[-1,-1], 64:[-1,0], 128:[-1,1]}
print("Created dictionaries for navigating NumPy arrays")

# Taking some measurements of the flow direction and river cell rasters, for the dispersal_area function
raster_origin = Raster('flowdir').extent.upperLeft
raster_cell_dim = Raster('flowdir').meanCellWidth

# Copying ECd raster values to a table format
arcpy.gp.Sample_sa("ECd_TP", "ECd_TP", "ECd_TPtable.dbf", "NEAREST", "VALUE")
arcpy.gp.Sample_sa("ECd_TN", "ECd_TN", "ECd_TNtable.dbf", "NEAREST", "VALUE")
print("ECd values, coordinates copied to tables")

# ArcMap table format converted to CSV
arcpy.TableToTable_conversion("ECd_TPtable.dbf", hotspots_rpath, "ECd_TP.csv")
arcpy.TableToTable_conversion("ECd_TNtable.dbf", hotspots_rpath, "ECd_TN.csv")
print("Maps of ECd values converted to CSV files")

##############################
# FUNCTIONS
##############################
# This is used within the hotspots function to sort the full table of ECd values & coordinates
# Returns the fifth item (column) of each row, which corresponds to ECd values
def take_fifth(row):
    return row[4]

# This function converts [latitude, longitude] coordinate pairs from decimal degrees to decimal minutes
def decimal_minutes(coords):
    coords = [i.split(",") for i in coords]
    newcoords = []
    for pair in coords:
        lat_deg = math.trunc(float(pair[0])) # Whole number degrees component taken directly from the decimal degrees value
        lat_min = round((float(pair[0])-lat_deg)*60, 3) # Decimal minutes component = (decimal point remainder from degrees)*60
        newlat = str(lat_deg)+' '+str(lat_min)+'\' N' # New value = (whole number degrees) + (decimal minutes), combined as strings
        long_deg = abs(math.trunc(float(pair[1])))
        long_min = round((abs(float(pair[1]))-long_deg)*60, 3)
        newlong = str(long_deg)+' '+str(long_min)+'\' W'
        newcoords.append([newlat, newlong])
    newcoords = [i[0]+', '+i[1] for i in newcoords]
    return newcoords

# This function takes CSV files of ECd (kg/ha/yr) values and latitude/longitude coordinates and returns:
# (1) List of ECd values, decimal minutes coords (string), decimal degrees coords (string) for each hotspot of interest
# (2) List of decimal degrees longitudes (float), decimal degrees latitudes (float) for each hotspot of interest
# Item 1 is used to create Table 2 in Chapter 2 of the GLRI Forest Planning report
# Items 1 & 2 are used together to create the GIS shapefiles & rasters identifying the hotspots of interest and their contributing/dispersal areas
def hotspots(ECd_file):
    with open(ECd_file) as f:
        lines = f.readlines()
    rawdata = [line.split(',') for line in lines[1:]]
    ECds = np.array([i[4] for i in rawdata], dtype=np.float_) # ECd values (kg/ha/yr) correspond to last column in ECd_file
    data = []
    # Extracting the subset of ECd, latitude, and longitude values corresponding to hotspots
    hotspots_counter = 1
    while hotspots_counter <= number_of_hotspots: # Until values have been extracted for the requisite number of hotspots:
        print("Extracting values for hotspot "+str(hotspots_counter)+" out of "+str(number_of_hotspots))
        hotspot_index = np.where(ECds == np.max(ECds))[0][0] # Look up index of the current maximum ECd
        hotspot_row = rawdata[hotspot_index] # Identify the rawdata row holding values for the same hotspot
        data.append(hotspot_row) # Add the rawdata row to the subset list
        del rawdata[hotspot_index] # Remove the current maximum ECd from rawdata
        ECds = np.delete(ECds, hotspot_index) # Remove the current maximum ECd from the ECds array
        hotspots_counter += 1
    rawdata = [] # Clear variable to free the memory
    hotspots_ECds = [float(i[4]) for i in data] # Dynamic export coefficients (ECd, kg/ha/yr) for the hotspots of interest
    ECds = np.append(ECds, hotspots_ECds) # Restore ECds array to its full list of values by appending the ones previously removed
    # Determining the percentile of the smallest ECd value for all hotspots
    percentiles = range(99,0,-1)
    lowest_ECd = min(hotspots_ECds) # Smallest ECd value for a hotspot being considered
    for val in percentiles: # Working in decreasing order from 99th to 1st percentile:
        this_ECd = np.percentile(ECds, val) # Calculate the ECd value at exactly the nth precentile, where n = val
        if this_ECd < lowest_ECd: # If ECd value at the current nth percentile is less than the smallest ECd value for any hotspot of interest, then all hotspot ECd values can be said to represent values at or above the nth percentile
            output_percentile = val
            this_ECd = 0.0 # Clear variable to free the memory
            break
        this_ECd = 0.0 # Clear variable to free the memory
    output_median = np.median(ECds)
    output_SD = np.std(ECds) # Standard deviation
    ECds = np.array([]) # Clear variable to free the memory
    hotspots_lats = [i[3] for i in data] # Latitudes (decimal degrees) correspond to "Y" values in ECd_file
    hotspots_longs = [i[2] for i in data] # Longitudes (decimal degrees) correspond to "X" values in ECd_file
    latlong_degrees = [x+','+y for x,y in zip(hotspots_lats,hotspots_longs)] # Decimal degree coords merged together
    latlong_minutes = decimal_minutes(latlong_degrees) # Decimal minute coords merged together
    return [[hotspots_ECds,latlong_minutes,latlong_degrees],[[float(x),float(y)] for x,y in zip(hotspots_longs,hotspots_lats)]]

# This function takes a Pandas dataframe of hotspot ranks, coordinates, and ECd values and 
# creates a raster identifying the contributing areas of each hotspot
def contributing_area(hotspots_dataframe, pollutant):
    CA_cells = [] # List of NumPy array cells contributing to any hotspot
    for rank in [i for i in hotspots_dataframe['Rank']]:
        print("Constructing contributing area for hotspot "+str(rank)+" out of "+str(number_of_hotspots))
        check_cells = [] # List of cells that still need to be checked for their surrounding flowdir_array values
        hotspot_long_lat = [hotspots_dataframe.iloc[rank-1,]['X'], hotspots_dataframe.iloc[rank-1,]['Y']] # Merging a hotspot's longitude (X) and latitude (Y) values into a two-item list
        # Using hotspot_long_lat to find the corresponding starting row & column in check_cells
        hotspot_cell_row = abs(int((raster_origin.Y - hotspot_long_lat[1]) / raster_cell_dim)) # Rows from origin = difference in latitude (Y) / size of raster cells
        hotspot_cell_col = abs(int((raster_origin.X - hotspot_long_lat[0]) / raster_cell_dim)) # Cols from origin = difference in longitude (X) / size of raster cells
        check_cells.append([hotspot_cell_row, hotspot_cell_col]) # Making the hotspot cell the first cell to be checked
        CA_cells.append(hotspot_long_lat)
        while len(check_cells) > 0: # Loop stops once all relevant cells have had their surrounding flowdir_array values checked
            current_cell = check_cells[0]
            current_row = current_cell[0]
            current_col = current_cell[1]
            # Cells in check_cells are checked for which neighboring cells (if any) have flowdir_array values that would cause water to flow from the neighbor cell to the current cell
            # Neighbors: list of [orientation, flowdir_array value] pairs
            # neighbor_orientation values: 1 for the cell to the immediate right of the current cell; add 1 for each move clockwise through neighbor cells
            neighbors = [[1,flowdir_array[current_row, current_col+1]], [2,flowdir_array[current_row+1, current_col+1]], [3,flowdir_array[current_row+1, current_col]], [4,flowdir_array[current_row+1, current_col-1]], [5,flowdir_array[current_row, current_col-1]], [6,flowdir_array[current_row-1, current_col-1]], [7,flowdir_array[current_row-1, current_col]], [8,flowdir_array[current_row-1, current_col+1]]]
            for neighbor_cell in neighbors:
                neighbor_orientation = neighbor_cell[0]
                neighbor_flowdir = neighbor_cell[1]
                if neighbor_flowdir == ca_flowdir_dict[neighbor_orientation]: # If neighbor_cell's orientation and flowdir_array values line up such that water would flow from neighbor_cell to current_cell:
                    # Calculate neighbor_cell's coordinates
                    neighbor_row = current_row + ca_neighbor_dict[neighbor_orientation][0]
                    neighbor_col = current_col + ca_neighbor_dict[neighbor_orientation][1]
                    neighbor_long = hotspot_long_lat[0] + ((neighbor_col-hotspot_cell_col)*raster_cell_dim)
                    neighbor_lat = hotspot_long_lat[1] - ((neighbor_row-hotspot_cell_row)*raster_cell_dim)
                    # Add neighbor_cell coordinates to the lists they are not already on
                    if neighbor_cell not in check_cells:
                        check_cells.append([neighbor_row, neighbor_col])
                    if neighbor_cell not in CA_cells:
                        CA_cells.append([neighbor_long, neighbor_lat])
            check_cells.remove(current_cell) # Remove current_cell so it is not checked again
    CA_points = [] # List of ArcPy point objects, each corresponding to a cell from CA_cells
    for coord_pair in CA_cells:
        new_point = arcpy.Point(coord_pair[0], coord_pair[1])
        CA_points.append(new_point)
    CA_array = arcpy.Array(CA_points) # Converting the list of point objects into an ArcPy array of points
    CA_multipoint = arcpy.Multipoint(CA_array, arcpy.SpatialReference(4269)) # Converting the array object into an ArcPy multipoint object, set to GCS_North_American_1983 projection
    arcpy.CopyFeatures_management(CA_multipoint, 'CA_'+pollutant+'_pts.shp') # Multipoint obect copied to a shapefile
    arcpy.FeatureToRaster_conversion('CA_'+pollutant+'_pts.shp', "Id", 'CA_'+pollutant, 'river') # Multipoint shapefile converted to a raster called CA_TP or CA_TN

# This function takes a Pandas dataframe of hotspot ranks, coordinates, and ECd values and 
# creates a raster identifying the dispersal areas of each hotspot
def dispersal_area(hotspots_dataframe, pollutant):
    DAs = {} # Dictionary holding all DA cell coordinates for all hotspots (i.e. {(hotspot rank):[list of [lat,long] pairs]})
    for rank in [i for i in hotspots_dataframe['Rank']]:
        print("Constructing dispersal area for hotspot "+str(rank)+" out of "+str(number_of_hotspots))
        DAs[rank] = [] # List of cells making up the dispersal area of the current hotspot
        current_long_lat = [hotspots_dataframe.iloc[rank-1,]['X'], hotspots_dataframe.iloc[rank-1,]['Y']] # Merging the hotspot cell longitude (X) and latitude (Y) values into a two-item list
        # Use hotspot [long, lat] to find the starting column, row in flowdir_array -> starting flowdir value
        cell_row = abs(int((raster_origin.Y - current_long_lat[1]) / raster_cell_dim)) # Rows from origin = difference in latitude (Y) / size of raster cells
        cell_col = abs(int((raster_origin.X - current_long_lat[0]) / raster_cell_dim)) # Cols from origin = difference in longitude (X) / size of raster cells
        # This variable checks to see if the current cell is a river cell
        # Even if the hotspot is located in a river cell, that cell will be included in the dispersal area calculation so at least one raster cell is highlighted for each hotspot's dispersal area
        river_cell = 1
        while river_cell == 1: # Loop stops once a river cell (raster value = 0) is reached
            DAs[rank].append(current_long_lat) # Add current cell coords to the list of DA cells
            current_flowdir = flowdir_array[cell_row, cell_col] # Flow direction value of current cell
            cell_change = da_flowdir_dict[current_flowdir] # [change in row, change in column] value pair, based on where water would flow from current cell
            cell_row += cell_change[0]
            cell_col += cell_change[1]
            current_long_lat = [current_long_lat[0]+(cell_change[1]*raster_cell_dim), current_long_lat[1]-(cell_change[0]*raster_cell_dim)] # current_long_lat values altered to reflect the new cell row & column
            river_cell = river_array[cell_row, cell_col] # Checking the river raster cell corresponding to the new cell row and column
    lines = [] # List of ArcPy polyline objects, each corresponding to all dispersal area cells for a hotspot
    for rank in range(1, len(DAs)+1):
        DA_coords = DAs[rank] # List of [long,lat] coordinate pairs
        DA_points = [] # List of ArcPy point objects corresponding to the coordinate pairs in DA_coords
        for coord_pair in DA_coords:
            new_point = arcpy.Point(coord_pair[0], coord_pair[1]) # Creating new ArcPy point object
            DA_points.append(new_point)
        new_array = arcpy.Array(DA_points) # Converting the list of point objects into an ArcPy array of points
        new_line = arcpy.Polyline(new_array, arcpy.SpatialReference(4269)) # Converting the array object into an ArcPy polyline object, set to GCS_North_American_1983 projection
        lines.append(new_line)
    arcpy.CopyFeatures_management(lines, 'DA_'+pollutant+'_line.shp') # Polyline obects merged together and copied to a shapefile
    arcpy.FeatureToRaster_conversion('DA_'+pollutant+'_line.shp', "Id", 'DA_'+pollutant, 'river') # Polyline shapefile converted to a raster called DA_TP or DA_TN

##############################
# RUNNING FUNCTIONS AND SAVING OUTPUT
##############################
print("Extracting subset of TP hotspots values:")
TP_subset = hotspots(hotspots_rpath+'\\ECd_TP.csv')
print("Extracting subset of TN hotspots values:")
TN_subset = hotspots(hotspots_rpath+'\\ECd_TN.csv')

TP_hotspots_cols = ['Rank', 'X', 'Y', 'ECd_kg/ha/yr'] # Column names for TP_hotspots
TP_hotspots = pd.DataFrame(columns = TP_hotspots_cols) # Pandas dataframe of hotspot rank, decimal degree coordinates, & ECd value for TP hotspots of interest
TP_hotspots['Rank'] = range(1,number_of_hotspots+1)
TP_hotspots['X'] = [i[0] for i in TP_subset[1]]
TP_hotspots['Y'] = [i[1] for i in TP_subset[1]]
TP_hotspots['ECd_(kg/ha/yr)'] = TP_subset[0][0]

TP_hotspots.to_csv(hotspots_rpath+'\\TP_htspts.csv', index=False) # Pandas dataframe saved as CSV
arcpy.TableToTable_conversion('TP_htspts.csv', hotspots_rpath, 'TP_htspts.dbf') # CSV table converted to ArcMap table
arcpy.MakeXYEventLayer_management('TP_htspts.dbf', 'X', 'Y', 'TP_lyr', arcpy.SpatialReference(4269)) # Longitudes (X) and latitudes (Y) from ArcMap table converted to a feature layer, set to GCS_North_American_1983 projection
arcpy.CopyFeatures_management('TP_lyr', 'TP_hotspots') # Feature layer copied to the final point shapefile of TP hotspots locations
print("TP hotspots points generated")

TN_hotspots_cols = ['Rank', 'X', 'Y', 'ECd_kg/ha/yr'] # Column names for TN_hotspots
TN_hotspots = pd.DataFrame(columns = TN_hotspots_cols) # Pandas dataframe of hotspot rank, decimal degree coordinates, & ECd value for TN hotspots of interest
TN_hotspots['Rank'] = range(1,number_of_hotspots+1)
TN_hotspots['X'] = [i[0] for i in TN_subset[1]]
TN_hotspots['Y'] = [i[1] for i in TN_subset[1]]
TN_hotspots['ECd_(kg/ha/yr)'] = TN_subset[0][0]

TN_hotspots.to_csv(hotspots_rpath+'\\TN_htspts.csv', index=False) # Pandas dataframe saved as CSV
arcpy.TableToTable_conversion('TN_htspts.csv', hotspots_rpath, 'TN_htspts.dbf') # CSV table converted to ArcMap table
arcpy.MakeXYEventLayer_management('TN_htspts.dbf', 'X', 'Y', 'TN_lyr', arcpy.SpatialReference(4269)) # Longitudes (X) and latitudes (Y) from ArcMap table converted to a feature layer, set to GCS_North_American_1983 projection
arcpy.CopyFeatures_management('TN_lyr', 'TN_hotspots') # Feature layer copied to the final point shapefile of TN hotspots locations
print("TN hotspots points generated")

print("Generating TP hotspots contributing areas:")
contributing_area(TP_hotspots, "TP")
print("Generating TN hotspots contributing areas:")
contributing_area(TN_hotspots, "TN")
print("Hotspots contributing areas generated")

print("Generating TP hotspots dispersal areas:")
dispersal_area(TP_hotspots, "TP")
print("Generating TN hotspots dispersal areas:")
dispersal_area(TN_hotspots, "TN")
print("Hotspots dispersal areas generated")

table2_cols = ['Pollutant','Rank','ECd (kg/ha/yr)','Lat, long (decimal minutes)','Lat, long (decimal degrees)'] # Column names for table2
table2 = pd.DataFrame(columns = table2_cols) # Pandas dataframe of pollutant name, hotspot rank, ECd value, and hotspot coordinates (decimal minutes and decimal degrees) for all hotspots of interest
table2['Pollutant'] = ['TP']*number_of_hotspots + ['TN']*number_of_hotspots
table2['Rank'] = range(1,number_of_hotspots+1)*2
table2['ECd (kg/ha/yr)'] = TP_subset[0][0] + TN_subset[0][0]
table2['Lat, long (decimal minutes)'] = TP_subset[0][1] + TN_subset[0][1]
table2['Lat, long (decimal degrees)'] = TP_subset[0][2] + TN_subset[0][2]

table2.to_csv(hotspots_rpath+'\\table2.csv', index=False) # Pandas dataframe saved as CSV
print("Output table of hotspots coordinates and ECd values generated at: "+hotspots_rpath+'\\table2.csv')

##############################
# CLEANING UP FILES AND VARIABLES
##############################
# Deleting some imtermediary data to avoid them being confused for final results
arcpy.Delete_management("dem")
arcpy.Delete_management("ec_tp_wtd")
arcpy.Delete_management("ec_tn_wtd")
arcpy.Delete_management("fill")
arcpy.Delete_management("flowdir")
arcpy.Delete_management("flowaccum")
arcpy.Delete_management("ECd_TPtable.dbf")
arcpy.Delete_management("ECd_TNtable.dbf")
os.remove(hotspots_rpath+'\\ECd_TP.csv')
os.remove(hotspots_rpath+'\\ECd_TN.csv')
arcpy.Delete_management("CA_TP_pts.shp")
arcpy.Delete_management("CA_TN_pts.shp")
arcpy.Delete_management("DA_TP_line.shp")
arcpy.Delete_management("DA_TN_line.shp")
arcpy.Delete_management("TP_htspts.csv")
arcpy.Delete_management("TP_htspts.dbf")
arcpy.Delete_management("TP_lyr")
arcpy.Delete_management("TN_htspts.csv")
arcpy.Delete_management("TN_htspts.dbf")
arcpy.Delete_management("TN_lyr")

# Replacing the ArcGIS Spatial Analyst license
arcpy.CheckInExtension("Spatial")
